﻿using System.Web.Mvc;
using Nop.Plugin.Widgets.EKomi.Models;
using Nop.Services.Configuration;
using Nop.Web.Framework.Controllers;
using System.Net;
using System;
using System.IO;
using System.Collections.Generic;
using Nop.Core.Caching;
using Nop.Core.Infrastructure;
using Nop.Plugin.Widgets.EKomi.Services;
using System.Linq;

namespace Nop.Plugin.Widgets.EKomi.Controllers
{
    public class WidgetsEKomiController : Controller
    {
        private readonly ISettingService _settingService;
        private readonly IEKomiService _eKomiService;

        internal const string FEEDBACK_ALL_KEY = "Nop.eKomi.feedback";
        internal const string FEEDBACK_WIDGET_FILTER_KEY = "Nop.eKomi.feedback.widgetfilter-{0}";

        public WidgetsEKomiController(ISettingService settingService,
            IEKomiService eKomiService)
        {
            this._settingService = settingService;
            this._eKomiService = eKomiService;
        }

        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult Configure()
        {
            var model = new ConfigurationModel();
            model.InterfaceId = _settingService.GetSettingByKey<string>("Widgets.EKomi.InterfaceId", "00000");
            model.InterfacePassword = _settingService.GetSettingByKey<string>("Widgets.EKomi.InterfacePassword", "000000000000000");

            return View("Nop.Plugin.Widgets.EKomi.Views.WidgetsEKomi.Configure", model);
        }

        [HttpPost]
        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult Configure(ConfigurationModel model)
        {
            if (!ModelState.IsValid)
                return Configure();

            //save settings
            _settingService.SetSetting("Widgets.EKomi.InterfaceId", model.InterfaceId);
            _settingService.SetSetting("Widgets.EKomi.InterfacePassword", model.InterfacePassword);

            return View("Nop.Plugin.Widgets.EKomi.Views.WidgetsEKomi.Configure", model);
        }

        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult ConfigureWidget(int widgetId)
        {
            var model = new WidgetConfigurationModel();
            if (widgetId > 0)
            {
                model.DisplayScript = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayScript.{0}", widgetId));
                model.WidgetScript = _settingService.GetSettingByKey<string>(string.Format("Widgets.EKomi.WidgetScript.{0}", widgetId));
                model.DisplayFeedback = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayFeedback.{0}", widgetId));
                model.DisplayRating = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayRating.{0}", widgetId));
                model.DisplayStars = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayStars.{0}", widgetId));
                model.CommentsNumber = _settingService.GetSettingByKey<int>(string.Format("Widgets.EKomi.CommentsNumber.{0}", widgetId));
            }
            else
            {
                model.DisplayScript = false;
                model.WidgetScript = @"<script type=""text/javascript"">
var eKomiWGProt = ((""https:"" == document.location.protocol) ? ""https://"" : ""http://"");
document.write(unescape(""%3Cscript src='"" + eKomiWGProt + ""connect.ekomi.de/widget/xyzxyzxyzxyzxyz.js' type='text/javascript'%3E%3C/script%3E""));
</script>";
                model.DisplayFeedback = false;
                model.DisplayRating = false;
                model.DisplayStars = false;
                model.CommentsNumber = 3;
            }

            return View("Nop.Plugin.Widgets.EKomi.Views.WidgetsEKomi.ConfigureWidget", model);
        }

        [HttpPost]
        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult ConfigureWidget(int widgetId, WidgetConfigurationModel model)
        {
            if (!ModelState.IsValid)
                return ConfigureWidget(widgetId);

            //save settings
            _settingService.SetSetting(string.Format("Widgets.EKomi.DisplayScript.{0}", widgetId), model.DisplayScript);
            _settingService.SetSetting(string.Format("Widgets.EKomi.WidgetScript.{0}", widgetId), model.WidgetScript);
            _settingService.SetSetting(string.Format("Widgets.EKomi.DisplayFeedback.{0}", widgetId), model.DisplayFeedback);
            _settingService.SetSetting(string.Format("Widgets.EKomi.DisplayRating.{0}", widgetId), model.DisplayRating);
            _settingService.SetSetting(string.Format("Widgets.EKomi.DisplayStars.{0}", widgetId), model.DisplayStars);
            _settingService.SetSetting(string.Format("Widgets.EKomi.CommentsNumber.{0}", widgetId), model.CommentsNumber);

            return View("Nop.Plugin.Widgets.EKomi.Views.WidgetsEKomi.ConfigureWidget", model);
        }

        [ChildActionOnly]
        public ActionResult PublicInfo(int widgetId)
        {
            var model = new PublicInfoModel();
            var displayScript = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayScript.{0}", widgetId));
            var widgetScript = _settingService.GetSettingByKey<string>(string.Format("Widgets.EKomi.WidgetScript.{0}", widgetId));
            var displayComments = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayFeedback.{0}", widgetId));
            var displayRating = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayRating.{0}", widgetId));
            var displayStars = _settingService.GetSettingByKey<bool>(string.Format("Widgets.EKomi.DisplayStars.{0}", widgetId));
            var commentsNumber = _settingService.GetSettingByKey<int>(string.Format("Widgets.EKomi.CommentsNumber.{0}", widgetId));


            model.DisplayFeedback = displayComments;
            model.DisplayRating = displayRating;
            model.DisplayStars = displayStars;
            if (displayComments)
            {
                model.Feedback = _eKomiService
                    .GetRandom(widgetId, commentsNumber, 5, 30)
                    .Select(x =>
                    {
                        var subCatModel = new FeedbackModel()
                        {
                            Comment = Nop.Core.Html.HtmlHelper.ConvertPlainTextToHtml(x.Comment),
                            Rating = x.Rating
                        };
                        return subCatModel;
                    })
                    .ToArray();

                //http://www.santiane.fr/santiane-avis.htm
            }

            model.DisplayScripts = displayScript;
            if (displayScript)
            {
                model.WidgetScript = widgetScript;
            }


            return View("Nop.Plugin.Widgets.EKomi.Views.WidgetsEKomi.PublicInfo", model);
        }

      
    }
}